/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.gui;

import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;

import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JRadioButton;
import javax.swing.SwingConstants;

import ch.cern.lhcb.xmleditor.specific.ElementName;
import ch.cern.lhcb.xmleditor.specific.XmlDisplay;

/**
 * A dialog windows that allows configuration of the displaying of the name of
 * nodes
 * 
 * @author Sebastien Ponce
 */
public class NameDialog extends AbstractDialog {

    /**
     * the layout manager of the panel
     */
    private GridBagLayout layout;

    /**
     * the name of the item in the combo box for PCDATAS
     */
    private static final String TEXT_NODE_NAME = "#Plain Text";

    /**
     * the name of the item in the combo box for Comments
     */
    private static final String COMMENT_NAME = "#Comments";

    /**
     * constructor : builds the dialog
     * 
     * @param owner the frame where this dialog should appear
     * @param xmlDisplay the XmlDisplay that must be used
     */
    public NameDialog(Frame owner, XmlDisplay xmlDisplay) {
        // calls the super constructor
        super(owner, "Name Displaying", xmlDisplay);
        // adds a layout to the panel where details are displayed
        layout = new GridBagLayout();
        panel.setLayout(layout);
        // adds new items to the combo box for text Nodes and comments
        box.addItem(TEXT_NODE_NAME);
        box.addItem(COMMENT_NAME);
    }

    /**
     * builds the panel for a given element name of a given XmlDisplay by
     * displaying some subelements
     * 
     * @param name the name of the element for which the panel should be built
     * @param xmlDisplay the XmlDisplay dealing with the dtd in which the
     *        element name was declared
     */
    protected void buildPanel(String name, XmlDisplay xmlDisplay) {
        // says whether we are dealing with text nodes or comments
        boolean isTextNode = name.equals(TEXT_NODE_NAME);
        boolean isComment = name.equals(COMMENT_NAME);
        // stores the current xmlDisplay
        this.xmlDisplay = xmlDisplay;
        // this will contain the attribute names and the way to display this
        // element
        String[] attributes;
        ElementName elementName;
        // if we deal with the text node
        if (isTextNode) {
            // no attributes
            attributes = new String[] {};
            // gets the ElementName
            elementName = xmlDisplay.getTextNodeName();
        } else if (isComment) {
            // no attributes
            attributes = new String[] {};
            // gets the ElementName
            elementName = xmlDisplay.getCommentName();
        } else {
            // else we deal with a real element
            // retrieves the attributes
            attributes = xmlDisplay.getAttributeNames(name);
            // retrieves the current ElementName
            elementName = xmlDisplay.getElementName(name);
        }
        // creates a GridBagConstraint object that will be used for display
        GridBagConstraints c = new GridBagConstraints();
        // makes title for the unique column
        JLabel label = new JLabel("Select name source");
        Font font = new Font("dialog", Font.PLAIN, 16);
        label.setFont(font);
        label.setAlignmentX(SwingConstants.RIGHT);
        c.gridx = 0;
        c.gridy = 0;
        c.anchor = GridBagConstraints.WEST;
        c.insets = new Insets(0, 0, 10, 0);
        layout.setConstraints(label, c);
        panel.add(label);
        // creates a radiobutton group
        ButtonGroup buttonGroup = new ButtonGroup();
        // the default name possibility : the type of the element
        JRadioButton typeRadioButton = new JRadioButton("Element type");
        typeRadioButton.setActionCommand("Type");
        buttonGroup.add(typeRadioButton);
        if (elementName.nameType == ElementName.TYPE_NAME) {
            typeRadioButton.setSelected(true);
        }
        typeRadioButton.addActionListener(this);
        c.gridy += 1;
        c.insets = new Insets(0, 20, 0, 0);
        layout.setConstraints(typeRadioButton, c);
        panel.add(typeRadioButton);
        // the value possibility. Exists if we deal we a text node or a comment
        if (isTextNode || isComment || xmlDisplay.elementHasValue(name)) {
            JRadioButton valueRadioButton = new JRadioButton("Element Value");
            valueRadioButton.setActionCommand("Value");
            buttonGroup.add(valueRadioButton);
            if (elementName.nameType == ElementName.VALUE_NAME) {
                valueRadioButton.setSelected(true);
            }
            valueRadioButton.addActionListener(this);
            c.gridy += 1;
            layout.setConstraints(valueRadioButton, c);
            panel.add(valueRadioButton);
        }
        // for each attribute, displays it
        for (int i = 0; i < attributes.length; i++) {
            // creates and display radiobutton
            JRadioButton radioButton = new JRadioButton(attributes[i]);
            radioButton.setActionCommand("Attribute : " + attributes[i]);
            buttonGroup.add(radioButton);
            if (elementName.nameType == ElementName.ATTRIBUTE_NAME
                    && elementName.attributeName.equals(attributes[i])) {
                radioButton.setSelected(true);
            }
            radioButton.addActionListener(this);
            c.gridy += 1;
            layout.setConstraints(radioButton, c);
            panel.add(radioButton);
        }
    }

    /**
     * called each time an action event occurs
     * 
     * @param e the ActionEvent
     */
    public void actionPerformed(ActionEvent e) {
        super.actionPerformed(e);
        //  if the event comes from a radiobutton
        if (e.getSource() instanceof JRadioButton) {
            JRadioButton source = (JRadioButton) e.getSource();
            // the new ElementName
            ElementName elementName;
            // computes its value
            if (source.getActionCommand() == "Type") {
                // if the source is the typeRadioButton
                elementName = new ElementName(ElementName.TYPE_NAME, "");
            } else if (source.getActionCommand() == "Value") {
                // if the source is the valueRadioButton
                elementName = new ElementName(ElementName.VALUE_NAME, "");
            } else {
                // else the source is a radiobutton whose name is an attribute
                // name
                String attrName = source.getActionCommand();
                attrName = attrName.substring(12);
                elementName = new ElementName(ElementName.ATTRIBUTE_NAME,
                        attrName);
            }
            // The name of the selected element in the combobox
            String name = (String) box.getSelectedItem();
            if (name.equals(TEXT_NODE_NAME)) {
                // if we deal with text nodes
                xmlDisplay.setTextNodeName(elementName);
            } else if (name.equals(COMMENT_NAME)) {
                // if we deal with text nodes
                xmlDisplay.setCommentName(elementName);
            } else {
                // else we deal with en element
                xmlDisplay.setElementName(name, elementName);
            }
        }
    }

}