/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.gui;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;

import org.w3c.dom.Node;

import ch.cern.lhcb.xmleditor.specific.DtdFile;
import ch.cern.lhcb.xmleditor.specific.Setup;
import ch.cern.lhcb.xmleditor.specific.XmlDisplay;
import ch.cern.lhcb.xmleditor.specific.XmlFile;
import ch.cern.lhcb.xmleditor.xerces.DocumentImpl;
import ch.cern.lhcb.xmleditor.xerces.ElementImpl;
import ch.cern.lhcb.xmleditor.xerces.TreeElement;

/**
 * this class extends the default treeNode in two ways. First, it allows the
 * node to have children that are not displayed at a given moment. Then, it
 * allows the node to keep the history of the place it occupies in the tree, by
 * pointing on its predecessor.
 * 
 * @author Sebastien Ponce
 */
public class XmlTreeNode extends DefaultMutableTreeNode {

    /**
     * the associated Node
     */
    private TreeElement node;

    /**
     * a link to the node that was previously displayed in place of this one.
     * null if there was no previous node
     */
    private XmlTreeNode oldNode;

    /**
     * a link to the node that took the place of this one. null if this node is
     * still displayed
     */
    private XmlTreeNode newNode;

    /**
     * the tree to wich this node belongs null if it does not belong to a tree
     */
    private XmlTree jTree;

    /**
     * default constructor
     */
    public XmlTreeNode() {
        // calls the super constructor
        super();
        // initializes the fields
        oldNode = null;
        node = null;
        jTree = null;
    }

    /**
     * Constructor of the class
     * 
     * @param node the DOM node of wich this TreeNode is the representation
     * @param jTree the tree to which this new node will belong
     */
    public XmlTreeNode(TreeElement node, XmlTree jTree) {
        // calls the super constructor
        super();
        // initializes the fields
        this.node = node;
        this.jTree = jTree;
        oldNode = null;
        // adds this new TreeNode to the list of representation of node
        node.addTreeNode(this);
    }

    /**
     * returns the Node associated with this XmlTreeNode
     * 
     * @return the Node associated with this XmlTreeNode
     */
    public TreeElement getNode() {
        return node;
    }

    /**
     * says whether the child number index is displayed or not
     * 
     * @param index the index of the child concerned
     * @return true if it is displayed
     */
    private boolean isChildDisplayed(int index) {
        // gets the child
        Node child = ((XmlTreeNode) super.getChildAt(index)).getNode();
        // analyses its type
        switch (child.getNodeType()) {
        case Node.COMMENT_NODE:
            // these are comments
            return jTree.getTreeView().isCommentDisplayed();
        case Node.ELEMENT_NODE:
            // these are subelements
            // deals with the root node first since it is not known by the dtd
            if (this.getNode().getNodeType() == Node.DOCUMENT_NODE) { return true; }
            // else, this is a regular node
            // a an XmlDisplay is available, it has the answer, else the node is
            // displayed
            XmlDisplay xmlDisplay = this.getXmlDisplay();
            if (xmlDisplay != null) {
                return xmlDisplay.isSubElementDisplayed(node.getNodeName(),
                        child.getNodeName());
            } else {
                return true;
            }
        case Node.ENTITY_REFERENCE_NODE:
            // Entity references are always displayed
            return true;
        case Node.TEXT_NODE:
            // these are values
            return !jTree.getTreeView().isValueDisplayedAsAttribute();
        }
        return false;
    }

    /**
     * says whether the name of this node is editable or not
     * 
     * @return true if the name of this node is editable
     */
    public boolean isNameEditable() {
        XmlDisplay xmlDisplay = this.getXmlDisplay();
        if (xmlDisplay != null) {
            return xmlDisplay.isNameEditable(this.getNode());
        } else {
            return false;
        }
    }

    /**
     * overrides the default method to hide the node that we don't want to
     * display
     * 
     * @return the number of displayed children
     */
    public int getChildCount() {
        // deals with invalid node
        if (node == null) { return 0; }
        // calls the ancestor method
        int result = 0;
        // looks at every node to count those we want to display
        for (int i = 0; i < super.getChildCount(); i++) {
            if (this.isChildDisplayed(i)) {
                result++;
            }
        }
        // returns the result
        return result;
    }

    /**
     * get the real index of a child this real index is the index among all
     * children of this node while index is supposed to be the index among the
     * displayed nodes
     * 
     * @param index the index among displayed nodes
     * @return the index among all children nodes
     */
    private int getRealIndex(int index) {
        // First, if there are no children, returns 0
        if (super.getChildCount() == 0) { return 0; }
        // else initializes the result
        int result = 0;
        // for each node, see if it should be counted or not
        while (index > -1 && result < super.getChildCount()) {
            // if this node is displayed, then it was counted in index
            if (this.isChildDisplayed(result)) {
                index--;
            }
            result++;
        }
        // if index is not -1, index exceeds the number of children, returns
        // childCount
        if (index > -1) {
            return super.getChildCount();
        } else {
            // else returns the result. Beware that we did increment once more
            // than needed
            return result - 1;
        }
    }

    /**
     * overrides the default method to take care of non-displayed nodes
     * 
     * @param childIndex the index of the child to retrieve among displayed
     *        nodes
     * @return the node at childIndex
     */
    public TreeNode getChildAt(int childIndex) {
        int result = getRealIndex(childIndex);
        return super.getChildAt(result);
    }

    /**
     * overrides the default method to catch the new Node name and store it
     * Actually, this method is called at the end of the edition of the node
     * name, with a String as userObject which is the new node name
     * 
     * @param userObject the new user object, here the new node name
     */
    public void setUserObject(Object userObject) {
        XmlDisplay xmlDisplay = this.getXmlDisplay();
        if (xmlDisplay != null) {
            // sets the new name
            xmlDisplay.setNodeName(node, (String) userObject);
            // validates the change
            node.nodeNameChanged();
        }
        // ends any compound edit going on
        jTree.getTreeView().getEditor().getUndoManager().endAnyInnerCompound();
    }

    /**
     * overrides the default method to display a custom node name
     * 
     * @return the name to be displayed for this node
     */
    public String toString() {
        // deals with invalid node
        if (node == null) {
            return "";
        } else {
            // else tries to call the XmlDisplay
            XmlDisplay xmlDisplay = this.getXmlDisplay();
            if (xmlDisplay != null) {
                return xmlDisplay.getNodeName(node);
            } else {
                // if no XmlDisplay is available, puts the default name
                return node.getNodeName();
            }
        }
    }

    /**
     * sets the value of the node attached to this TreeNode. TO BE USED VERY
     * CAREFULLY, ONLY WHEN YOU PRECISELY KNOW WHAT YOU ARE DOING
     * 
     * @param node the new node attached to this treenode
     */
    public void setNode(TreeElement node) {
        this.node = node;
    }

    /**
     * says whether the reference mechanism is on in the dtd for this node
     */
    public boolean allowsReferences() {
        if (node == null) { return false; }
        DocumentImpl doc = (DocumentImpl) node.getOwnerDocument();
        if (doc == null) { return false; }
        XmlFile xml = doc.getXmlFile();
        if (xml == null) { return false; }
        XmlDisplay display = xml.getXmlDisplay();
        if (display == null) { return false; }
        DtdFile dtd = display.getDtdFile();
        if (dtd == null) { return false; }
        Setup setup = jTree.getTreeView().getEditor().getSetup();
        if (setup == null) { return false; }
        return setup.allowsReferences(dtd.getName());
    }

    /**
     * says whether this node is a reference or not. If references are not
     * allowed for the DTD corresponding to this node, always returns false
     * 
     * @return true if this node is a reference
     */
    public boolean isReference() {
        if (!this.allowsReferences()) { return false; }
        if (node instanceof ElementImpl) {
            return ((ElementImpl) node).isReference();
        } else {
            return false;
        }
    }

    /**
     * places the oldNode field to oldNode
     * 
     * @param oldNode the new oldNode
     */
    public void setOldNode(XmlTreeNode oldNode) {
        this.oldNode = oldNode;
    }

    /**
     * returns the oldNode of this TreeNode
     * 
     * @return the oldNode
     */
    public XmlTreeNode getOldNode() {
        return oldNode;
    }

    /**
     * places the newNode field to newNode
     * 
     * @param newNode the new newNode
     */
    public void setNewNode(XmlTreeNode newNode) {
        this.newNode = newNode;
    }

    /**
     * returns the newNode of this TreeNode
     * 
     * @return the newNode
     */
    public XmlTreeNode getNewNode() {
        return newNode;
    }

    /**
     * overrides the default method to take care of not displayed nodes
     * 
     * @param node the node we want to get the index
     * @return the index of node
     */
    public int getIndex(TreeNode node) {
        int result = super.getIndex(node);
        for (int i = result; i >= 0; i--) {
            if (!this.isChildDisplayed(i)) {
                result--;
            }
        }
        return result;
    }

    /**
     * inserts a new node in this tree at childIndex
     * 
     * @param newChild the node to insert
     * @param childIndex its position
     * @return whether it is displayed or not at this moment
     */
    public boolean insertNode(MutableTreeNode newChild, int childIndex) {
        int realIndex = getRealIndex(childIndex);
        super.insert(newChild, realIndex);
        return this.isChildDisplayed(realIndex);
    }

    /**
     * overrides the default method to take care of non-displayed nodes
     * 
     * @param childIndex the index of the child to remove among displayed nodes
     */
    public void remove(int childIndex) {
        // remove its parent from the child
        ((XmlTreeNode) getChildAt(childIndex)).setParent(null);
        // removes the child from its parent
        children.removeElementAt(getRealIndex(childIndex));
    }

    /**
     * replaces a node in this tree at childIndex
     * 
     * @param newChild the node to insert in place of the old one
     * @param childIndex the position where to replace the node
     */
    public void replaceNode(MutableTreeNode newChild, int childIndex) {
        // gets the real index
        int realIndex = getRealIndex(childIndex);
        // removes the node, and update it
        XmlTreeNode treeNode = (XmlTreeNode) super.getChildAt(realIndex);
        treeNode.setParent(null);
        children.removeElementAt(realIndex);
        // puts the new node
        super.insert(newChild, realIndex);
    }

    /**
     * gets the XmlTree displaying this Node
     * 
     * @return the XmlTree displaying this node
     */
    public XmlTree getXmlTree() {
        return jTree;
    }

    /**
     * gets the XmlDisplay associated with this TreeNode
     * 
     * @return the XmlDisplay associated with this TreeNode
     */
    public XmlDisplay getXmlDisplay() {
        return XmlTree.getXmlDisplayFromNode(this.getNode());
    }

    /**
     * returns the parent of this XmlTreeNode
     * 
     * @return the parent of this TreeNode
     */
    public XmlTreeNode getParentTreeNode() {
        return (XmlTreeNode) super.getParent();
    }

    /**
     * says whether node has a representation that is an ancestor of this node
     * or not. Note that a given node is considered as an ancestor of itself.
     * 
     * @param node the node to be tested
     * @return true if the current node is a descendant of a representation of
     *         node
     */
    public boolean isNodeAncestor(TreeElement node) {
        if (this.node == node) { return true; }
        XmlTreeNode parent = (XmlTreeNode) this.getParent();
        if (parent == null) { return false; }
        return parent.isNodeAncestor(node);
    }

    /**
     * returns the index of the first child of this TreeNode containing the Node
     * node or the Node pointed to by node if node appears to be a reference. If
     * none, returns -1
     * 
     * @param node the searched node
     * @return its index or -1
     */
    public int getChildIndex(Node node) {
        // loop over the children and try to find one with the right node
        // inside. This node could be directly inside or could be the
        // reference that was opened to get the node inside.
        int childCount = this.getChildCount();
        for (int index = 0; index < childCount; index++) {
            XmlTreeNode child = (XmlTreeNode) this.getChildAt(index);
            if (child.getNode() == node
                    || ((child.getOldNode() != null) && (child.getOldNode()
                            .getNode() == node))) { return index; }
        }
        return -1;
    }

}