/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.xerces;

import java.util.HashSet;
import java.util.Iterator;

import org.w3c.dom.Node;

import ch.cern.lhcb.xmleditor.gui.XmlTreeNode;
import ch.cern.lhcb.xmleditor.specific.XmlFile;

/**
 * This class subclasses the default AttrImpl implementation to take some
 * displaying facts into account
 * 
 * @author Sebastien Ponce
 */
class AttrImpl extends org.apache.xerces.dom.AttrImpl implements TreeElement {

    /**
     * this is the set of TreeNodes associated with this attribute Actually the
     * treeNodes listed here are those of the ownerElement of this attribute
     * since an attribute is only displayed through its owner element (at least
     * currently)
     */
    private HashSet treeNodes;

    protected AttrImpl(DocumentImpl ownerDocument, String name) {
        super(ownerDocument, name);
        treeNodes = new HashSet();
    }

    protected AttrImpl() {
        super();
        treeNodes = new HashSet();
    }

    /**
     * Returns a duplicate of this node.
     * 
     * @param deep if true, performs a deep copy
     */
    public Node cloneNode(boolean deep) {
        AttrImpl result = (AttrImpl) super.cloneNode(deep);
        result.treeNodes = new HashSet();
        return result;
    }

    /**
     * this method should be called whenever the name of this node is edited
     */
    public void nodeNameChanged() {
        // nothing to do since never called
        // This would only be called if attributes were displayed in the tree,
        // which is not the case today
    }

    /**
     * updates the upToDate flag of the file corresponding to this node
     */
    private void updateFile() {
        XmlFile file = ((DocumentImpl) this.getOwnerDocument()).getXmlFile();
        if (file != null) {
            file.upToDate = false;
        }
    }

    /**
     * this method should be called whenever the value of this node is edited
     */
    public void nodeValueChanged() {
        // updates the upToDate field of the corresponding file
        this.updateFile();
        // updates the different displays of this node's ownerElement
        Iterator iterator = treeNodes.iterator();
        while (iterator.hasNext()) {
            XmlTreeNode treeNode = (XmlTreeNode) iterator.next();
            // redisplays the whole tree to validate the changes
            // treeNode.getXmlTree().getTreeView().redisplayAll();
        }
    }

    /**
     * this method should be called when one representation of this node is
     * removed
     * 
     * @param treeNode the representation to be removed. No check done on this
     *        parameter. I MUST be non-null and representing this node we are in
     */
    private void internalRepresentationRemoved(XmlTreeNode treeNode) {
        // removes treeNode from the list of representations for this node
        this.removeTreeNode(treeNode);
        // redisplays the whole tree to validate the changes
        treeNode.getXmlTree().getTreeView().redisplayAll();
    }

    /**
     * this method should be called when this node is removed
     */
    public void removed() {
        // updates the upToDate field of the corresponding file
        this.updateFile();
        // updates the different representations of this node's ownerElement
        while (!treeNodes.isEmpty()) {
            // gets the treeNode
            XmlTreeNode treeNode = (XmlTreeNode) treeNodes.iterator().next();
            // removes it
            internalRepresentationRemoved(treeNode);
        }
    }

    /**
     * adds treeNode to the list of treeNode associated with this node
     * 
     * @param treeNode the TreeNode to add to the list
     */
    public void addTreeNode(XmlTreeNode treeNode) {
        // Actually the treeNodes listed here are those of the ownerElement of
        // this attribute since an attribute is only displayed through its owner
        // element (at least currently)
        treeNodes.add(treeNode);
    }

    /**
     * removes treeNode from the list of treeNode associated with this node
     * 
     * @param treeNode the TreeNode to remove from the list
     */
    public void removeTreeNode(XmlTreeNode treeNode) {
        // Actually the treeNodes listed here are those of the ownerElement of
        // this attribute since an attribute is only displayed through its owner
        // element (at least currently)
        treeNodes.remove(treeNode);
    }

    /**
     * says whether node is an ancestor of this node or not Take care that the
     * notion of ancestor does include the parent-child relation as well as the
     * element-attribute one. This means that an element is ancestor of its
     * attributes.
     * 
     * @param node the node to be tested
     * @return true if the current node is a descendant of node
     */
    public boolean isNodeAncestor(TreeElement node) {
        if (node == this) { return true; }
        TreeElement parent = (TreeElement) this.getOwnerElement();
        if (parent == null) {
            return false;
        } else {
            return parent.isNodeAncestor(node);
        }
    }
}
