/*
 * com.simulogic.cese -- Cell Electrophysiology Simulation Environment (CESE)
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 * 
 * Dalhousie Univ., Halifax, NS, Canada
 *
 * BeanBuilder code Copyright (C) 2002 Mark Davidson
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.cese.properties;

import java.awt.Component;
import java.beans.PropertyEditor;
import java.util.Hashtable;

import javax.swing.JCheckBox;
import javax.swing.JTable;
import javax.swing.UIManager;
import javax.swing.beaninfo.SwingEditorSupport;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import com.simulogic.cese.ResourceManager;

/**
 * Column model for the PropertyTable
 */
public class PropertyColumnModel extends DefaultTableColumnModel {

	private final static String COL_LABEL_PROP = ResourceManager
			.getResource("Name");

	private final static String COL_LABEL_DESC = ResourceManager
			.getResource("Description");

	private final static String COL_LABEL_VALUE = ResourceManager
			.getResource("Value");

	private final static String COL_LABEL_CLAMP = ResourceManager
			.getResource("Clamp");

	private static final int minColWidth = 50;

	/**
	 * Default constructor
	 */
	public PropertyColumnModel() {
		// Configure the columns and add them to the model
		TableColumn column;

		// Property
		column = new TableColumn(PropertyTableModel.COL_NAME);
		column.setHeaderValue(COL_LABEL_PROP);
		column.setPreferredWidth(minColWidth);
		column.setCellRenderer(new PropertyNameRenderer());
		addColumn(column);

		// Value
		column = new TableColumn(PropertyTableModel.COL_VALUE);
		column.setHeaderValue(COL_LABEL_VALUE);
		column.setPreferredWidth(minColWidth);
		column.setCellEditor(new PropertyValueEditor());
		column.setCellRenderer(new PropertyValueRenderer());
		addColumn(column);

		// Clamp
		column = new TableColumn(PropertyTableModel.COL_CLAMP);
		column.setHeaderValue(COL_LABEL_CLAMP);
		column.setPreferredWidth(minColWidth / 2);
		column.sizeWidthToFit();
		column.setResizable(false);
		column.setCellEditor(new PropertyClampEditor());
		column.setCellRenderer(new PropertyClampRenderer());

		addColumn(column);
	}

	/**
	 * Renders the name of the clamp property.
	 */
	class PropertyClampRenderer extends DefaultTableCellRenderer {

		/**
		 * Get UI for current editor, including custom editor button if
		 * applicable.
		 */
		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column) {

			JCheckBox chb = new JCheckBox(((ClampAttribute) value)
					.getVariable(), ((ClampAttribute) value).isClamped());

			return chb;
		}
	}

	/**
	 * Renders the name of the property. Sets the short description of the
	 * property as the tooltip text.
	 */
	class PropertyNameRenderer extends DefaultTableCellRenderer {

		/**
		 * Get UI for current editor, including custom editor button if
		 * applicable.
		 */
		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column) {

			PropertyTableModel model = (PropertyTableModel) table.getModel();
			ModelPropertyDescriptor desc = model
					.getModelPropertyDescriptor(row);

			setToolTipText(desc.getDescriptor().getShortDescription());

			Component renderer = super.getTableCellRendererComponent(table,
					value, isSelected, hasFocus, row, column);

			if (desc.isChanged()) {
				renderer.setBackground(java.awt.Color.yellow);
			} else {
				renderer.setBackground(UIManager.getColor("Text.background"));
			}

			return renderer;
		}
	}

	/**
	 * Renderer for a value with a property editor or installs the default cell
	 * rendererer.
	 */
	class PropertyValueRenderer implements TableCellRenderer {

		private DefaultTableCellRenderer renderer;

		private PropertyEditor editor;

		private Hashtable editors;

		private Class type;

		/**
		 * Default constructor
		 */
		public PropertyValueRenderer() {
			renderer = new DefaultTableCellRenderer();
			editors = new Hashtable();
		}

		/**
		 * Get UI for current editor, including custom editor button if
		 * applicable.
		 */
		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column) {

			PropertyTableModel model = (PropertyTableModel) table.getModel();
			type = model.getPropertyType(row);

			if (type != null) {
				editor = (PropertyEditor) editors.get(type);
				if (editor == null) {
					editor = model.getPropertyEditor(row);

					if (editor != null) {
						editors.put(type, editor);
					}
				}
			} else {
				editor = null;
			}

			if (editor != null) {
				// Special case for the enumerated properties. Must reinitialize
				// to reset the combo box values.
				if (editor instanceof SwingEditorSupport) {
					((SwingEditorSupport) editor).init(model
							.getModelPropertyDescriptor(row).getDescriptor());
				}

				editor.setValue(value);

				Component comp = editor.getCustomEditor();
				if (comp != null) {
					comp.setEnabled(isSelected);
					return comp;
				}
			}
			return renderer.getTableCellRendererComponent(table, value,
					isSelected, hasFocus, row, column);
		}

		/**
		 * Retrieves the property editor for this value
		 * 
		 * @return property editor
		 */
		public PropertyEditor getPropertyEditor() {
			return editor;
		}
	}

}